package com.sitemesh;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;

/**
 * The Page object wraps the contents of the original (undecorated) page.
 *
 * <p>The original data in its entirity can be written using the writePage()
 * methods. It may also contain a set of properties - these vary among
 * different {@link com.sitemesh.PageParser} implementations.</p>
 *
 * <p>Typically a Page is no use to a {@link com.sitemesh.Decorator} as it needs
 * specific details relevant to the content-type of that page (<i>e.g.</i> HTML
 * pages). The appropriate {@link com.sitemesh.PageParser} is responsible
 * for returning extended implementations of pages such as {@link com.sitemesh.HTMLPage}
 * which are of more use to the Decorator. New media types (<i>e.g.</i> WML) could
 * be added to the system by extending Page and implementing an appropriate PageParser.</p>
 *
 * @author <a href="mailto:joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.8 $
 */
public interface Page {

	/**
	 * Write the entire contents of the <code>Page</code>, in the format before
	 * it was parsed, to the <code>OutputStream</code>.
	 *
	 * @param out OutputStream to write to.
	 * @exception java.io.IOException Rethrown if cannot write to stream.
	 *
	 * @see #writePage(java.io.Writer)
	 */
	void writePage( OutputStream out ) throws IOException;

	/**
	 * Write the entire contents of the <code>Page</code>, in the format before
	 * it was parsed, to the <code>Writer</code>.
	 *
	 * @param out Writer to write to.
	 * @exception java.io.IOException Rethrown if cannot write to writer.
	 *
	 * @see #writePage(java.io.OutputStream)
	 */
	void writePage( Writer out ) throws IOException;

	/**
	 * Length of the <code>Page</code>, in the format before
	 * it was parsed.
	 *
	 * @return Length of page data (in number of bytes).
	 */
	int getContentLength();

	/**
	 * Get a property embedded into the <code>Page</code> as a <code>String</code>.
	 *
	 * @param name Name of property
	 * @return Property value
	 */
	String getProperty( String name );

	/**
	 * Get a property embedded into the <code>Page</code> as an <code>int</code>.
	 * Returns 0 if property not specified or not valid number.
	 *
	 * @param name Name of property
	 * @return Property value
	 */
	int getIntProperty( String name );

	/**
	 * Get a property embedded into the <code>Page</code> as a <code>long</code>.
	 * Returns 0L if property not specified or not valid number.
	 *
	 * @param name Name of property
	 * @return Property value
	 */
	long getLongProperty( String name );

	/**
	 * Get a property embedded into the <code>Page</code> as a <code>boolean</code>.
	 * Returns true if value starts with '1', 't' or 'y' (case-insensitive) -
	 * otherwise returns false.
	 *
	 * @param name Name of property
	 * @return Property value
	 */
	boolean getBooleanProperty( String name );

	/**
	 * Get properties embedded into the <code>Page</code> as a <code>String[]</code>.
	 * Returns zero length array if no properties set.
	 *
	 * @param name Name of property
	 * @return Property values
	 */
	String[] getProperties( String name );

	/**
	 * Determine whether a property embedded into the <code>Page</code> has been set.
	 *
	 * @param name Name of property
	 * @return Whether it has been set
	 */
	boolean isPropertySet( String name );

	/**
	 * Determine the number of properties embedded into the <code>Page</code> with
	 * a given name.
	 *
	 * @param name Name of property
	 * @return Number of properties set with given name.
	 */
	int getPropertySize( String name );

	/**
	 * Get all available property keys for the <code>Page</code>.
	 *
	 * @return Property keys
	 */
	String[] getPropertyKeys();

	/**
	 * Determine whether this <code>Page</code> can be cached or not. If allowed to
	 * be cached for a specified period of time, <code>true</code> should be initially
	 * returned, and repeated calls to this method will determine whether the page
	 * is still allowed to remain cached.
	 */
	boolean shouldCache();

	/**
	 * Return the request of the original page.
	 */
	HttpServletRequest getRequest();

	/**
	 * Create snapshot of Request. Subsequent modifications to the request by
	 * the servlet container will not be returned by {@link #getRequest()}
	 */
	void setRequest(HttpServletRequest request);

}
