package com.sitemesh.parser;

import com.sitemesh.Page;
import java.io.*;
import java.util.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
 * Abstract implementation of {@link com.sitemesh.Page} .
 *
 * <p>Contains base methods for storing and accessing page properties.
 * Also stores {@link #pageData} as byte[] and implements write???()
 * methods.</p>
 *
 * <p>Concrete implementations need only set the {@link #pageData} and
 * call {@link #addProperty(java.lang.String,java.lang.String)} to
 * add all the required information.</p>
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.7 $
 *
 * @see com.sitemesh.Page
 */
public abstract class AbstractPage implements Page {

	/**
	 * Map of all properties.
	 * Key is String. Value is java.util.List of multiple String values.
	 */
	private Map properties = new HashMap();

	/**
	 * Date of page contents.
	 */
	protected byte[] pageData = new byte[0];

	/**
	 * RequestURI of original Page.
	 */
	public HttpServletRequest request;

	public void writePage( OutputStream out ) throws IOException {
		out.write( pageData );
		out.flush();
	}

	public void writePage( Writer out ) throws IOException {
		Reader r = new InputStreamReader( new ByteArrayInputStream( pageData ) );
		while ( true ) {
			int i = r.read();
			if ( i == -1 ) break;
			out.write(i);
		}
	}

	public int getContentLength() {
		return pageData.length;
	}

	public String getProperty( String name ) {
		if ( !isPropertySet( name ) ) return null;
		Iterator i = ( ( List )properties.get( name ) ).iterator();
		if ( i.hasNext() ) return ( String )i.next();
		return null;
	}

	public int getIntProperty( String name ) {
		try {
			return Integer.parseInt( noNull( getProperty( name ) ) );
		}
		catch ( NumberFormatException e ) {
			return 0;
		}
	}

	public long getLongProperty( String name ) {
		try {
			return Long.parseLong( noNull( getProperty( name ) ) );
		}
		catch ( NumberFormatException e ) {
			return 0;
		}
	}

	public boolean getBooleanProperty( String name ) {
		String property = getProperty( name );
		if ( property == null ) return false;
		switch ( property.charAt( 0 ) ) {
			case '1':
			case 't':
			case 'T':
			case 'y':
			case 'Y':
				return true;
			default:
				return false;
		}
	}

	public String[] getProperties( String name ) {
		String[] result   = new String[getPropertySize( name )];
		List propertyList = ( List )properties.get( name );
		Iterator it       = propertyList.iterator();
		int i             = 0;
		while ( it.hasNext() ) {
			result[i++] = ( String )it.next();
		}
		return result;
	}

	public boolean isPropertySet( String name ) {
		return properties.containsKey( name );
	}

	public int getPropertySize( String name ) {
		if ( !isPropertySet( name ) ) return 0;
		return ( ( List )properties.get( name ) ).size();
	}

	public String[] getPropertyKeys() {
		synchronized( properties ) {
			Set keys        = properties.keySet();
			String[] result = new String[keys.size()];
			Iterator it     = keys.iterator();
			int i           = 0;
			while ( it.hasNext() ) {
				result[i++] = ( String )it.next();
			}
			return result;
		}
	}

	/**
	 * Returns false. Override as necessary.
	 *
	 * @see com.sitemesh.Page#shouldCache()
	 */
	public boolean shouldCache() {
		return false;
	}

	/**
	 * @see com.sitemesh.Page#getRequest()
	 */
	public HttpServletRequest getRequest() {
		return request;
	}

	/**
	 * Create snapshot of Request.
	 *
	 * @see com.sitemesh.Page#getRequest()
	 */
	public void setRequest(HttpServletRequest request) {
		this.request = new PageRequest(request);
	}

	/**
	 * Add a property to the properties list.
	 *
	 * <p>Duplicate keys are allowed... property will be given
	 * multiple values. Sub-classes should use this for adding
	 * properties to the page.</p>
	 *
	 * @param name Name of property
	 * @param value Value of property
	 */
	protected void addProperty( String name, String value ) {
		List propertyValues;
		if ( properties.containsKey( name ) ) {
			propertyValues = ( List )properties.get( name );
		}
		else {
			propertyValues = new ArrayList();
			properties.put( name, propertyValues );
		}
		propertyValues.add( value );
	}

	/**
	 * Return String as is, or "" if null. (Prevents NullPointerExceptions)
	 */
	protected String noNull( String in ) {
		return in == null ? "" : in;
	}

}


class PageRequest extends HttpServletRequestWrapper {

	private String requestURI, method, pathInfo, pathTranslated,
	               queryString, servletPath;

	public PageRequest(HttpServletRequest request) {
		super(request);
		requestURI = request.getRequestURI();
		method = request.getMethod();
		pathInfo = request.getPathInfo();
		pathTranslated = request.getPathTranslated();
		queryString = request.getQueryString();
		servletPath = request.getServletPath();
	}

	public String getRequestURI() {
		return requestURI;
	}

	public String getMethod() {
		return method;
	}

	public String getPathInfo() {
		return pathInfo;
	}

	public String getPathTranslated() {
		return pathTranslated;
	}

	public String getQueryString() {
		return queryString;
	}

	public String getServletPath() {
		return servletPath;
	}

}
