package com.sitemesh.parser;

import java.io.IOException;
import java.io.Writer;
import org.apache.xml.serialize.HTMLSerializer;
import org.apache.xml.serialize.Method;
import org.apache.xml.serialize.OutputFormat;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Implementation of {@link com.sitemesh.HTMLPage} that populates itself
 * from a parsed DOM document.
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.7 $
 */
public class DOMPage extends AbstractHTMLPage {

	/**
	 * BODY element.
	 */
	private HTMLBodyElement bodyTag;

	/**
	 * HEAD element.
	 */
	private HTMLHeadElement headTag;

	/**
	 * Entire Document.
	 */
	private HTMLDocument document;

	/**
	 * Create instance and set all initial properties and data.
	 *
	 * @param pageData Original page data
	 * @param document Parsed DOM HTML document.
	 */
	public DOMPage( byte[] pageData, HTMLDocument document ) {
		this.document = document;
		this.pageData = pageData;
		HTMLHtmlElement htmlTag = ( HTMLHtmlElement )document.getDocumentElement();
		addAttributeList( null, htmlTag.getAttributes() );
		addProperty( "title", document.getTitle() );
		bodyTag = ( HTMLBodyElement )document.getBody();
		addAttributeList( "body", bodyTag.getAttributes() );
		{
			NodeList htmlChildren = htmlTag.getChildNodes();
			for ( int i = 0; i < htmlChildren.getLength(); i++ ) {
				Node curr = htmlChildren.item( i );
				if ( curr instanceof HTMLHeadElement ) {
					headTag = ( HTMLHeadElement )curr;
					break;
				}
				if ( curr instanceof HTMLBodyElement ) break;
			}
		}
		if ( headTag != null ) {
			NodeList metaTags = headTag.getChildNodes();
			for ( int i = 0; i < metaTags.getLength(); i++ ) {
				Node curr = metaTags.item( i );
				if ( curr instanceof HTMLMetaElement ) {
					HTMLMetaElement metaTag = ( HTMLMetaElement )curr;
					String name             = metaTag.getName();
					String content          = metaTag.getContent();
					if ( name != null && name.length() > 0 && content != null && content.length() > 0 ) {
						addProperty( "meta." + metaTag.getName(), metaTag.getContent() );
					}
				}
			}
		}
	}

	/**
	 * Iterate through the attributes of an element and add them as properties.
	 *
	 * @param prefix Prefix to add to key name (without '.').
	 * @param attribute Attributes of element.
	 */
	private void addAttributeList( String prefix, NamedNodeMap attributes ) {
		if ( prefix == null ) prefix = "";
		if ( prefix.length() > 0 ) prefix += ".";
		for ( int i = 0; i < attributes.getLength(); i++ ) {
			Attr curr = ( Attr )attributes.item( i );
			if ( curr.getValue() != null && curr.getValue().length() > 0 ) {
				addProperty( prefix + curr.getName(), curr.getValue() );
			}
		}
	}

	/**
	 * Serialize an Element to a stream, but don't write outermost tags, only
	 * contents.
	 */
	private void serialize( Writer out, Element element ) throws IOException {
		OutputFormat format   = new OutputFormat( Method.HTML, null, false );
		DocumentFragment frag = document.createDocumentFragment();
		NodeList children     = element.getChildNodes();
		for ( int i = 0; i < children.getLength(); i++ ) {
			Node n = children.item( i );
			if (n instanceof HTMLTitleElement) continue;
			frag.appendChild( n.cloneNode( true ) );
		}
		HTMLSerializer serializer = new HTMLSerializer( out, format ) {
			public void serializeNode( Node n ) {
				_started = true;
				super.serializeNode( n );
			}
		};
		serializer.serialize( frag );
	}

	/**
	 * Serialize HEAD contents to stream.
	 */
	public void writeHead( Writer out ) throws IOException {
		serialize( out, headTag );
	}

	/**
	 * Serialize BODY contents to stream.
	 */
	public void writeBody( Writer out ) throws IOException {
		serialize( out, bodyTag );
	}

}
